@@ -323,12 +323,12 @@ class IpAddressInUse(InUse):
 
 
 class VlanIdInUse(InUse):
-    """An exception indicating VLAN creation failed because it's already in use.
+    """A network operational error indicating a VLAN ID is already in use.
 
     A specialization of the InUse exception indicating network creation failed
     because a specified VLAN is already in use on the physical network.
 
-    :param vlan_id: The LVAN ID.
+    :param vlan_id: The VLAN ID.
     :param physical_network: The physical network.
     """
     message = _("Unable to create the network. "
